from setuptools import setup


setup(
    name="django-friendships",
    version="1.0.2",
    author="Zoltan Kohalmy",
    author_email="zoltankohalmy@gmail.com",
    url="https://github.com/Aperyon/django-friendships",
    description="Simple solution for Django User Friendships",
    packages=['friendship']
)
